@@ -12,7 +12,7 @@ from __future__ import absolute_import, division, unicode_literals
 import re
 
 from mo_dots import split_field
-from mo_future import first, is_text, long, string_types, text
+from mo_future import first, is_text, string_types, text
 from mo_parsing import listwrap
 
 from mo_sql_parsing.keywords import RESERVED, join_keywords, precedence
@@ -119,8 +119,8 @@ unordered_clauses = [
 
 ordered_clauses = [
     "orderby",
-    "limit",
     "offset",
+    "limit",
 ]
 
 
@@ -138,8 +138,6 @@ class Formatter:
     _gte = Operator(">=")
     _lte = Operator("<=")
     _eq = Operator("=")
-    _in = Operator("in")
-    _nin = Operator("not in")
     _or = Operator("or")
     _and = Operator("and")
     _binary_and = Operator("&")
@@ -163,7 +161,7 @@ class Formatter:
 
     def dispatch(self, json, prec=100):
         if isinstance(json, list):
-            return self.sql_list(json)
+            return self.sql_list(json, prec=precedence["list"])
         if isinstance(json, dict):
             if len(json) == 0:
                 return ""
@@ -171,12 +169,16 @@ class Formatter:
                 return self.value(json, prec)
             elif "join" in json:
                 return self._join_on(json)
+            elif "insert" in json:
+                return self.insert(json)
             elif json.keys() & set(ordered_clauses):
                 return self.ordered_query(json, prec)
             elif json.keys() & set(unordered_clauses):
                 return self.unordered_query(json, prec)
             elif "null" in json:
                 return "NULL"
+            elif "trim" in json:
+                return self._trim(json, prec)
             else:
                 return self.op(json, prec)
         if isinstance(json, string_types):
@@ -295,6 +297,32 @@ class Formatter:
             self.dispatch(pair[0], precedence["collate"]), pair[1]
         )
 
+    def _in(self, json, prec):
+        member, set = json
+        if "literal" in set:
+            set = {"literal": listwrap(set["literal"])}
+        sql = (
+            self.dispatch(member, precedence["in"])
+            + " IN "
+            + self.dispatch(set, precedence["in"])
+        )
+        if prec < precedence["in"]:
+            sql = f"({sql})"
+        return sql
+
+    def _nin(self, json, prec):
+        member, set = json
+        if "literal" in set:
+            set = {"literal": listwrap(set["literal"])}
+        sql = (
+            self.dispatch(member, precedence["in"])
+            + " NOT IN "
+            + self.dispatch(set, precedence["in"])
+        )
+        if prec < precedence["in"]:
+            sql = f"({sql})"
+        return sql
+
     def _case(self, checks, prec):
         parts = ["CASE"]
         for check in checks if isinstance(checks, list) else [checks]:
@@ -325,7 +353,7 @@ class Formatter:
         type = self.dispatch(json[1], precedence["and"])
         return f"INTERVAL {amount} {type.upper()}"
 
-    def _literal(self, json, prec):
+    def _literal(self, json, prec=0):
         if isinstance(json, list):
             return "({0})".format(", ".join(
                 self._literal(v, precedence["literal"]) for v in json
@@ -335,6 +363,12 @@ class Formatter:
         else:
             return str(json)
 
+    def _get(self, json, prec):
+        v, i = json
+        v_sql = self.dispatch(v, prec=precedence["literal"])
+        i_sql = self.dispatch(i)
+        return f"{v_sql}[{i_sql}]"
+
     def _between(self, json, prec):
         return "{0} BETWEEN {1} AND {2}".format(
             self.dispatch(json[0], precedence["between"]),
@@ -342,6 +376,23 @@ class Formatter:
             self.dispatch(json[2], precedence["between"]),
         )
 
+    def _trim(self, json, prec):
+        c = json.get("characters")
+        d = json.get("direction")
+        v = json["trim"]
+        acc = ["TRIM("]
+        if d:
+            acc.append(d.upper())
+            acc.append(" ")
+        if c:
+            acc.append(self.dispatch(c))
+            acc.append(" ")
+        if c or d:
+            acc.append("FROM ")
+        acc.append(self.dispatch(v))
+        acc.append(")")
+        return "".join(acc)
+
     def _not_between(self, json, prec):
         return "{0} NOT BETWEEN {1} AND {2}".format(
             self.dispatch(json[0], precedence["between"]),
@@ -491,10 +542,47 @@ class Formatter:
         return f"ORDER BY {param}"
 
     def limit(self, json, prec):
-        return "LIMIT {0}".format(self.dispatch(json["limit"], precedence["order"]))
+        num = self.dispatch(json["limit"], precedence["order"])
+        return f"FETCH {num} ROWS ONLY"
 
     def offset(self, json, prec):
-        return "OFFSET {0}".format(self.dispatch(json["offset"], precedence["order"]))
+        num = self.dispatch(json["offset"], precedence["order"])
+        return f"OFFSET {num}"
+
+    def insert(self, json, prec=precedence["from"]):
+        acc = ["INSERT"]
+        if "overwrite" in json:
+            acc.append("OVERWRITE")
+        else:
+            acc.append("INTO")
+        acc.append(json["insert"])
+
+        if "columns" in json:
+            acc.append(self.sql_list(json))
+        if "values" in json:
+            values = json["values"]
+            if all(isinstance(row, dict) for row in values):
+                columns = list(sorted(set(k for row in values for k in row.keys())))
+                acc.append(self.sql_list(columns))
+                if "if exists" in json:
+                    acc.append("IF EXISTS")
+                acc.append("VALUES")
+                acc.append(",\n".join(
+                    "(" + ", ".join(self._literal(row[c]) for c in columns) + ")"
+                    for row in values
+                ))
+            else:
+                if "if exists" in json:
+                    acc.append("IF EXISTS")
+                acc.append("VALUES")
+                for row in values:
+                    acc.append("(" + ", ".join(self._literal(row)) + ")")
+
+        else:
+            if json["if exists"]:
+                acc.append("IF EXISTS")
+            acc.append(self.dispatch(json["query"]))
+        return " ".join(acc)
 
 
 setattr(Formatter, "with", Formatter.with_)
