@@ -9,16 +9,8 @@
 
 # SQL CONSTANTS
 from mo_parsing import *
-from mo_parsing.infix import delimited_list
 
-from mo_sql_parsing.utils import (
-    SQL_NULL,
-    int_num,
-    to_json_call,
-    ansi_string,
-    ansi_ident,
-    keyword,
-)
+from mo_sql_parsing.utils import SQL_NULL, keyword
 
 NULL = keyword("null") / (lambda: SQL_NULL)
 TRUE = keyword("true") / (lambda: True)
@@ -37,6 +29,7 @@ CREATE = keyword("create").suppress()
 CROSS = keyword("cross")
 DISTINCT = keyword("distinct")
 EXCEPT = keyword("except")
+FETCH = keyword("fetch").suppress()
 FROM = keyword("from").suppress()
 FULL = keyword("full")
 GROUP = keyword("group").suppress()
@@ -183,6 +176,7 @@ RESERVED = MatchFirst([
     ELSE,
     END,
     FALSE,
+    FETCH,
     FOREIGN,
     FROM,
     FULL,
@@ -286,6 +280,7 @@ precedence = {
     "or": 11,
     "lambda": 12,
     "join": 18,
+    "list": 18,
     "select": 30,
     "from": 30,
     "window": 35,
