@@ -9,6 +9,7 @@
 
 
 from mo_parsing.helpers import restOfLine
+from mo_parsing.infix import delimited_list
 from mo_parsing.whitespaces import NO_WHITESPACE, Whitespace
 
 from mo_sql_parsing.keywords import *
@@ -110,8 +111,14 @@ def parser(literal_string, ident, sqlserver=False):
             Group(
                 keyword("trim").suppress()
                 + LB
-                + expr("chars")
-                + Optional(FROM + expr("from"))
+                + Optional(
+                    (keyword("both") | keyword("trailing") | keyword("leading"))
+                    / (lambda t: t[0].lower())
+                )("direction")
+                + (
+                    assign("from", expr)
+                    | expr("chars") + Optional(assign("from", expr))
+                )
                 + RB
             ).set_parser_name("trim")
             / to_trim_call
@@ -407,6 +414,19 @@ def parser(literal_string, ident, sqlserver=False):
             + alias
         ).set_parser_name("table_source") / to_table
 
+        rows = Optional(keyword("row") | keyword("rows"))
+        limit = (
+            Optional(assign("offset", expr) + rows)
+            & Optional(
+                FETCH
+                + Optional(keyword("first") | keyword("next"))
+                + expr("limit")
+                + rows
+                + Optional(keyword("only"))
+            )
+            & Optional(assign("limit", expr))
+        )
+
         ordered_sql = (
             (
                 (unordered_sql | (LB + query + RB))
@@ -418,8 +438,7 @@ def parser(literal_string, ident, sqlserver=False):
                 )
             )("union")
             + Optional(ORDER_BY + delimited_list(Group(sort_column))("orderby"))
-            + Optional(LIMIT + expr("limit"))
-            + Optional(OFFSET + expr("offset"))
+            + limit
         ).set_parser_name("ordered sql") / to_union_call
 
         with_expr = delimited_list(Group(
@@ -551,16 +570,16 @@ def parser(literal_string, ident, sqlserver=False):
         )("drop")
 
         insert = (
-            keyword("insert")("op")
-            + (flag("overwrite") | keyword("into").suppress())
+            keyword("insert").suppress()
             + (
-                Optional(keyword("table").suppress())
-                + var_name("params")
-                + Optional(LB + delimited_list(var_name)("columns") + RB)
+                flag("overwrite") + keyword("table").suppress()
+                | keyword("into").suppress() + Optional(keyword("table").suppress())
             )
+            + var_name("table")
+            + Optional(LB + delimited_list(var_name)("columns") + RB)
             + Optional(flag("if exists"))
             + (values | query)("query")
-        ) / to_json_call
+        ) / to_insert_call
 
         update = (
             keyword("update")("op")
